﻿using System;
using System.Net;

namespace ProxyAuthenticationExample
{
    public class ProxyHelper
    {
        /// <summary>
        /// Does a check to see if the user has a proxy 
        /// and if they need to authenticate before connecting.
        /// </summary>
        /// <returns></returns>
        public bool RequiresAuthentication()
        {
            // See if the default credentials work.
            WebRequest.DefaultWebProxy.Credentials = CredentialCache.DefaultCredentials;
            bool authRequired = false;

            try
            {
                CheckExists();
            }
            catch (WebException webException)
            {
                if (((HttpWebResponse)webException.Response).StatusCode == HttpStatusCode.ProxyAuthenticationRequired)
                    authRequired = true;
            }

            return authRequired;
        }

        /// <summary>
        /// Once we have established that we need to authenticate
        /// we try a username and password / domain to 
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="password"></param>
        /// <param name="domain"></param>
        /// <returns></returns>
        public bool TryAuthenticate(string userName, 
            string password, string domain = null)
        {
            // Set the credentials
            WebRequest.DefaultWebProxy.Credentials = 
                new NetworkCredential(userName, password, domain);

            // Try and connect again
            try
            {
                CheckExists();
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        /// <summary>
        /// Check if we can connect by simply acessing a webpage.
        /// </summary>
        public void CheckExists()
        {
            // We need to check this by checking a site - this could be changed to anything.
            var request = (HttpWebRequest)WebRequest.Create("http://www.google.com/");
            request.Method = "HEAD";
            using (WebResponse webResponse = request.GetResponse())
            {
                // We only need to check if this throws an exception.
            }
        }
    }
}
